package com.hyp.web.controller.manager;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import com.hyp.common.core.domain.model.LoginUser;
import com.hyp.common.enums.ApplyStatusEnum;
import com.hyp.common.exception.ServiceException;
import com.hyp.common.utils.SecurityUtils;
import com.hyp.system.domain.RewApplyInfoList;
import com.hyp.system.domain.RewFileRelation;
import com.hyp.system.domain.dto.ApplyInfoListDTO;
import com.hyp.system.domain.dto.ApplyInfoListSaveDTO;
import com.hyp.system.domain.vo.ApplyInfoListVO;
import com.hyp.system.domain.vo.FileRelationVO;
import com.hyp.system.service.ISysDictDataService;
import com.hyp.system.service.RewApplyInfoListService;
import com.hyp.system.service.RewFileRelationService;
import com.hyp.system.service.RewScoreInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * packageName com.hyp.web.controller.manager
 *
 * @author wangxy
 * @version JDK 8
 * @className RewApplyInfoListManager
 * @date 2024/4/11
 * @description 申请
 */
@Component
public class ApplyInfoListManager {

    @Resource
    private RewApplyInfoListService applyInfoListService;

    @Resource
    private RewScoreInfoService scoreInfoService;
    
    @Resource
    private ISysDictDataService dictDataService;

    @Resource
    private RewFileRelationService relationService;

    /**
     * 
     *  奖项类型
     * @param null
     * @return 
     */
    
    private static final String REW_APPLY_TYPE = "rew_apply_type";


    public List<ApplyInfoListVO> selectApplyInfoList(ApplyInfoListDTO applyInfoListDTO){
        return applyInfoListService.selectApplyInfoList(applyInfoListDTO);
    }

    /**
     *
     *  生成applyId
     * @param applyType
     * @return com.hyp.system.domain.vo.ApplyInfoListVO
     */
    @Transactional(rollbackFor = Exception.class)
    public ApplyInfoListVO getApplyId(String applyType){
        LoginUser loginUser = SecurityUtils.getLoginUser();
         List<RewApplyInfoList> list = applyInfoListService.lambdaQuery()
                .eq(RewApplyInfoList::getApplyType, applyType)
                 .eq(RewApplyInfoList::getCreateId, loginUser.getUserId())
                 .list();
         if(CollUtil.isNotEmpty(list)){
             throw new ServiceException("该用户已申请，不能重复申请！");
         }
        RewApplyInfoList applyInfoList = new RewApplyInfoList();
        applyInfoList.setApplyType(applyType);

        String name = dictDataService.selectDictLabel(REW_APPLY_TYPE, applyType);
        if(StringUtils.isNotBlank(name)){
            applyInfoList.setApplyName(name);
        }
        applyInfoList.setApplyStatus(0);
        applyInfoList.setCreateId(loginUser.getUserId());
        applyInfoList.setCreateBy(loginUser.getUsername());
        applyInfoList.setUpdateTime(new Date());
        applyInfoList.setCreateTime(new Date());
        applyInfoListService.save(applyInfoList);
        return Convert.convert(ApplyInfoListVO.class, applyInfoList);
    }
    /**
     * 
     *  提交申请
     * @param applyInfoListSaveDTO
     * @return boolean
     */
    @Transactional(rollbackFor = Exception.class)
    public  boolean submit(ApplyInfoListSaveDTO applyInfoListSaveDTO){
        LoginUser loginUser = SecurityUtils.getLoginUser();
         RewApplyInfoList applyInfoList = applyInfoListService.getById(applyInfoListSaveDTO.getApplyId());
         if(Objects.nonNull(applyInfoList)){
             applyInfoList.setApplyStatus(applyInfoListSaveDTO.getApplyStatus());
             applyInfoList.setAppTime(new Date());
             applyInfoList.setCreateBy(loginUser.getUsername());
             applyInfoList.setIsReward(applyInfoListSaveDTO.getIsReward());
         }
        return applyInfoListService.saveOrUpdate(applyInfoList);
    }


    /**
     *
     *  删除申请
     * @param applyId
     * @return boolean
     */

    public  boolean deleteById(String applyId){
        relationService.lambdaUpdate()
                .eq(RewFileRelation::getApplyId, applyId)
                .remove();
        return applyInfoListService.removeById(applyId);
    }


    /**
     * 
     *  查询详细
     * @param applyId
     * @return com.hyp.system.domain.vo.ApplyInfoListVO
     */
    
    public ApplyInfoListVO detail(String applyId){
         RewApplyInfoList applyInfoList = applyInfoListService.getById(applyId);
         ApplyInfoListVO applyInfoListVO = Convert.convert(ApplyInfoListVO.class, applyInfoList);
         List<RewFileRelation> list = relationService.lambdaQuery().eq(RewFileRelation::getApplyId, applyId).list();
         List<FileRelationVO> fileRelationVOList = Convert.toList(FileRelationVO.class, list);
         if(CollUtil.isNotEmpty(fileRelationVOList)){
             applyInfoListVO.setFileRelationVOList(fileRelationVOList);
         }
        return applyInfoListVO;
        
    }

    /**
     * 
     *  审核提交
     * @param applyId
     * @return boolean
     */
    
    public  boolean audit(String applyId){
        return applyInfoListService.lambdaUpdate()
                .eq(RewApplyInfoList::getApplyId, applyId)
                .set(RewApplyInfoList::getApplyStatus, ApplyStatusEnum.AUDIT.getType())
                 .set(RewApplyInfoList::getAuthTime, new Date())
                 .update();
    }
    /**
     * 
     *  退回
     * @param applyId
     * @param applyMsg
     * @return boolean
     */
    
    public  boolean applyBack(String applyId,String applyMsg){
        return applyInfoListService.lambdaUpdate()
                .eq(RewApplyInfoList::getApplyId, applyId)
                .set(RewApplyInfoList::getApplyStatus, ApplyStatusEnum.BACK.getType())
                .set(Objects.nonNull(applyMsg), RewApplyInfoList::getApplyMsg, applyMsg)
                .set(RewApplyInfoList::getAuthTime, new Date())
                .update();
    }




}
