package com.hyp.framework.web.service;

import com.hyp.common.core.domain.entity.SysRole;
import com.hyp.system.domain.SysUserRole;
import com.hyp.system.mapper.SysUserRoleMapper;
import com.hyp.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import com.hyp.common.constant.CacheConstants;
import com.hyp.common.constant.Constants;
import com.hyp.common.constant.UserConstants;
import com.hyp.common.core.domain.entity.SysUser;
import com.hyp.common.core.domain.model.RegisterBody;
import com.hyp.common.core.redis.RedisCache;
import com.hyp.common.exception.user.CaptchaException;
import com.hyp.common.exception.user.CaptchaExpireException;
import com.hyp.common.utils.MessageUtils;
import com.hyp.common.utils.SecurityUtils;
import com.hyp.common.utils.StringUtils;
import com.hyp.framework.manager.AsyncManager;
import com.hyp.framework.manager.factory.AsyncFactory;
import com.hyp.system.service.ISysConfigService;
import com.hyp.system.service.ISysUserService;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 注册校验方法
 * 
 * @author ruoyi
 */
@Component
public class SysRegisterService
{
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private SysUserRoleMapper userRoleMapper;


    @Autowired
    private ISysRoleService roleService;

    /**
     * 注册
     */
    @Transactional(rollbackFor = Exception.class)
    public String register(RegisterBody registerBody)
    {
        String msg = "", username = registerBody.getUsername(), password = registerBody.getPassword();
        SysUser sysUser = new SysUser();
        sysUser.setUserName(username);

        // 验证码开关
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        if (captchaEnabled)
        {
            validateCaptcha(username, registerBody.getCode(), registerBody.getUuid());
        }

        if (StringUtils.isEmpty(username))
        {
            msg = "用户名不能为空";
        }
        else if (StringUtils.isEmpty(password))
        {
            msg = "用户密码不能为空";
        }
        else if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            msg = "账户长度必须在2到20个字符之间";
        }
        else if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            msg = "密码长度必须在5到20个字符之间";
        }
        else if (!userService.checkUserNameUnique(sysUser))
        {
            msg = "保存用户'" + username + "'失败，注册账号已存在";
        }
        else
        {
            sysUser.setNickName(username);
            sysUser.setPassword(SecurityUtils.encryptPassword(password));
            boolean regFlag = userService.registerUser(sysUser);
            //注册绑定角色
            roleBinding(sysUser,"yxxxj_xxl");
            if (!regFlag)
            {
                msg = "注册失败,请联系系统管理人员";
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.REGISTER, MessageUtils.message("user.register.success")));
            }
        }
        return msg;
    }

    /**
     * 
     *  注册绑定角色
     * @param sysUser
     * @param roleKey
     * @return void
     */
    
    public void roleBinding(SysUser sysUser,String roleKey) {
        //注册绑定角色
        userRoleMapper.deleteUserRoleByUserId(sysUser.getUserId());
        List<SysUserRole> list = new ArrayList<>();
        SysUserRole ur = new SysUserRole();
        ur.setUserId(sysUser.getUserId());
        SysRole sysRole = roleService.selectRoleByKey(roleKey);
        if(Objects.nonNull(sysRole)){
            ur.setRoleId(sysRole.getRoleId());
        }
        list.add(ur);
        userRoleMapper.batchUserRole(list);
    }

    /**
     * 校验验证码
     * 
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
        String captcha = redisCache.getCacheObject(verifyKey);
        redisCache.deleteObject(verifyKey);
        if (captcha == null)
        {
            throw new CaptchaExpireException();
        }
        if (!code.equalsIgnoreCase(captcha))
        {
            throw new CaptchaException();
        }
    }
}
